import os
import subprocess
from string import Template
import importlib.resources
from .. import templates as abies_templates


def cmake_cfg_cmd(
    src_dir: str = ".",
    build_dir: str = "build",
    cmake_args: list[str] = [],
    generator: str = "Ninja",
):
    """"""
    subprocess.check_call(
        ["cmake", "-G", generator, "-S", src_dir, "-B", build_dir] + cmake_args
    )


def cmake_build_cmd(build_dir: str = "build"):
    """"""
    subprocess.check_call(["cmake", "--build", build_dir])


def cmake_install_cmd(build_dir: str = "build", install_dir: str = "_install"):
    """"""
    subprocess.check_call(["cmake", "--install", build_dir, "--prefix", install_dir])


def build_verilator_model(verilog_file: str, build_root: str = "_abiesbuild"):
    """"""
    # c_include_dir = pkg_resources.resource_filename(__name__, "c_include")
    with importlib.resources.path("abies.simulator.c_include", "model_api.h") as p:
        c_include_dir = p.parent
    print(f"C Include: {c_include_dir}")
    # Verilog module and filename must match
    module_name = os.path.splitext(os.path.basename(verilog_file))[0]
    module_dir = os.path.dirname(verilog_file)
    module_src_dir = os.path.join(build_root, module_name)
    module_build_dir = os.path.join(module_src_dir, "build")

    os.makedirs(build_root, exist_ok=True)
    os.makedirs(module_src_dir, exist_ok=True)

    # Create CMakeLists.txt file
    cmake_args = {
        "module_name": module_name,
        "abies_c_include": c_include_dir,
        "module_src": os.path.abspath(verilog_file),
        "module_include": os.path.abspath(module_dir),
    }
    cmake_args_list = [f"-D{arg}={val}" for arg, val in cmake_args.items()]

    cmake_file = os.path.join(module_src_dir, "CMakeLists.txt")

    # Copy file.
    with importlib.resources.open_text(abies_templates, "model_cmake.txt") as fp:
        cmake_contents = fp.read()
    with open(cmake_file, "w") as cf:
        cf.write(cmake_contents)

    cpp_filt = {
        "model_name": f"V{module_name}",
    }
    # Create generated cpp file
    cpp_file = os.path.join(module_src_dir, f"{module_name}_model.cpp")
    with importlib.resources.open_text(abies_templates, "_model.cpp") as fp:
        cpp_t = Template(fp.read())
    cpp_contents = cpp_t.safe_substitute(cpp_filt)

    with open(cpp_file, "w") as fh:
        fh.write(cpp_contents)

    # Configure and build the library
    cmake_cfg_cmd(module_src_dir, module_build_dir, cmake_args_list)
    cmake_build_cmd(module_build_dir)
    cmake_install_cmd(module_build_dir, build_root)
